@@ -1428,16 +1428,18 @@ copyRawEvent(XGenericEventCookie *cookie_in,
     in = cookie_in->data;
 
     bits = count_bits(in->valuators.mask, in->valuators.mask_len);
-    len = sizeof(XIRawEvent) + in->valuators.mask_len;
+    len = pad_to_double(sizeof(XIRawEvent))
+        + pad_to_double(in->valuators.mask_len);
     len += bits * sizeof(double) * 2;
 
     ptr = cookie_out->data = malloc(len);
     if (!ptr)
         return False;
 
-    out = next_block(&ptr, sizeof(XIRawEvent));
+    out = next_block(&ptr, pad_to_double(sizeof(XIRawEvent)));
     *out = *in;
-    out->valuators.mask = next_block(&ptr, out->valuators.mask_len);
+    out->valuators.mask
+        = next_block(&ptr, pad_to_double(out->valuators.mask_len));
     memcpy(out->valuators.mask, in->valuators.mask, out->valuators.mask_len);
 
     out->valuators.values = next_block(&ptr, bits * sizeof(double));
@@ -1595,6 +1597,8 @@ wireToDeviceEvent(xXIDeviceEvent *in, XGenericEventCookie* cookie)
     len = sizeDeviceEvent(in->buttons_len * 4, in->valuators_len * 4, ptr);
 
     cookie->data = ptr_lib = malloc(len);
+    if (!ptr_lib)
+        return 0;
 
     out = next_block(&ptr_lib, sizeof(XIDeviceEvent));
     out->display = cookie->display;
@@ -1888,6 +1892,8 @@ wireToDeviceChangedEvent(xXIDeviceChangedEvent *in, XGenericEventCookie *cookie)
     len = size_classes((xXIAnyInfo*)&in[1], in->num_classes);
 
     cookie->data = out = malloc(sizeof(XIDeviceChangedEvent) + len);
+    if (!out)
+        return 0;
 
     out->type = in->type;
     out->serial = cookie->serial;
@@ -1918,7 +1924,9 @@ wireToHierarchyChangedEvent(xXIHierarchyEvent *in, XGenericEventCookie *cookie)
     xXIHierarchyInfo *info_in;
     XIHierarchyEvent *out;
 
-    cookie->data = out = malloc(sizeof(XIHierarchyEvent) + in->num_info * sizeof(XIHierarchyInfo));;
+    cookie->data = out = malloc(sizeof(XIHierarchyEvent) + in->num_info * sizeof(XIHierarchyInfo));
+    if (!out)
+        return 0;
 
     out->info           = (XIHierarchyInfo*)&out[1];
     out->display        = cookie->display;
@@ -1954,7 +1962,8 @@ wireToRawEvent(XExtDisplayInfo *info, xXIRawEvent *in, XGenericEventCookie *cook
     XIRawEvent *out;
     void *ptr;
 
-    len = sizeof(XIRawEvent) + in->valuators_len * 4;
+    len = pad_to_double(sizeof(XIRawEvent))
+        + pad_to_double(in->valuators_len * 4);
     bits = count_bits((unsigned char*)&in[1], in->valuators_len * 4);
     len += bits * sizeof(double) * 2; /* raw + normal */
 
@@ -1962,7 +1971,7 @@ wireToRawEvent(XExtDisplayInfo *info, xXIRawEvent *in, XGenericEventCookie *cook
     if (!ptr)
         return 0;
 
-    out = next_block(&ptr, sizeof(XIRawEvent));
+    out = next_block(&ptr, pad_to_double(sizeof(XIRawEvent)));
     out->type           = in->type;
     out->serial         = cookie->serial;
     out->display        = cookie->display;
@@ -1981,7 +1990,8 @@ wireToRawEvent(XExtDisplayInfo *info, xXIRawEvent *in, XGenericEventCookie *cook
         out->sourceid       = 0;
 
     out->valuators.mask_len = in->valuators_len * 4;
-    out->valuators.mask = next_block(&ptr, out->valuators.mask_len);
+    out->valuators.mask
+        = next_block(&ptr, pad_to_double(out->valuators.mask_len));
     memcpy(out->valuators.mask, &in[1], out->valuators.mask_len);
 
     out->valuators.values = next_block(&ptr, bits * sizeof(double));
@@ -2012,6 +2022,9 @@ wireToEnterLeave(xXIEnterEvent *in, XGenericEventCookie *cookie)
     len = sizeof(XIEnterEvent) + in->buttons_len * 4;
 
     cookie->data = out = malloc(len);
+    if (!out)
+        return 0;
+
     out->buttons.mask = (unsigned char*)&out[1];
 
     out->type           = in->type;
@@ -2056,6 +2069,8 @@ wireToPropertyEvent(xXIPropertyEvent *in, XGenericEventCookie *cookie)
     XIPropertyEvent *out = malloc(sizeof(XIPropertyEvent));
 
     cookie->data = out;
+    if (!out)
+        return 0;
 
     out->type           = in->type;
     out->serial         = cookie->serial;
@@ -2077,6 +2092,8 @@ wireToTouchOwnershipEvent(xXITouchOwnershipEvent *in,
     XITouchOwnershipEvent *out = malloc(sizeof(XITouchOwnershipEvent));
 
     cookie->data = out;
+    if (!out)
+        return 0;
 
     out->type           = in->type;
     out->serial         = cookie->serial;
@@ -2102,6 +2119,8 @@ wireToBarrierEvent(xXIBarrierEvent *in, XGenericEventCookie *cookie)
     XIBarrierEvent *out = malloc(sizeof(XIBarrierEvent));
 
     cookie->data = out;
+    if (!out)
+        return 0;
 
     out->display    = cookie->display;
     out->type       = in->type;
@@ -2133,6 +2152,8 @@ wireToPinchEvent(xXIGesturePinchEvent *in,
     XIGesturePinchEvent *out;
 
     cookie->data = out = malloc(sizeof(XIGesturePinchEvent));
+    if (!out)
+        return 0;
 
     out->display    = cookie->display;
     out->type       = in->type;
@@ -2178,6 +2199,8 @@ wireToSwipeEvent(xXIGestureSwipeEvent *in,
     XIGestureSwipeEvent *out;
 
     cookie->data = out = malloc(sizeof(XIGestureSwipeEvent));
+    if (!out)
+        return 0;
 
     out->display    = cookie->display;
     out->type       = in->type;
