@@ -67,15 +67,18 @@ XFeedbackState *
 XGetFeedbackControl(
     register Display	*dpy,
     XDevice		*dev,
-    int			*num_feedbacks)
+    int			*num_feedbacks_out)
 {
     XFeedbackState *Feedback = NULL;
     XFeedbackState *Sav = NULL;
     xFeedbackState *f = NULL;
     xFeedbackState *sav = NULL;
+    char *end = NULL;
     xGetFeedbackControlReq *req;
     xGetFeedbackControlReply rep;
     XExtDisplayInfo *info = XInput_find_display(dpy);
+    int num_feedbacks = 0;
+    int known_feedbacks = 0;
 
     LockDisplay(dpy);
     if (_XiCheckExtInit(dpy, XInput_Initial_Release, info) == -1)
@@ -94,7 +97,7 @@ XGetFeedbackControl(
 	size_t size = 0;
 	int i;
 
-	*num_feedbacks = rep.num_feedbacks;
+	num_feedbacks = rep.num_feedbacks;
 
 	if (rep.length < (INT_MAX >> 2)) {
 	    nbytes = rep.length << 2;
@@ -105,10 +108,12 @@ XGetFeedbackControl(
 	    goto out;
 	}
 	sav = f;
+	end = (char *)f + nbytes;
 	_XRead(dpy, (char *)f, nbytes);
 
-	for (i = 0; i < *num_feedbacks; i++) {
-	    if (f->length > nbytes)
+	for (i = 0; i < num_feedbacks; i++) {
+	    if ((char *)f + sizeof(*f) > end ||
+	        f->length == 0 || f->length > nbytes)
 		goto out;
 	    nbytes -= f->length;
 
@@ -125,6 +130,8 @@ XGetFeedbackControl(
 	    case StringFeedbackClass:
 	    {
 		xStringFeedbackState *strf = (xStringFeedbackState *) f;
+		if ((char *)f + sizeof(*strf) > end)
+		    goto out;
 		size += sizeof(XStringFeedbackState) +
 		    (strf->num_syms_supported * sizeof(KeySym));
 	    }
@@ -136,7 +143,7 @@ XGetFeedbackControl(
 		size += sizeof(XBellFeedbackState);
 		break;
 	    default:
-		size += f->length;
+		/* unknown classes are skipped */
 		break;
 	    }
 	    if (size > INT_MAX)
@@ -149,9 +156,11 @@ XGetFeedbackControl(
 	    goto out;
 
 	Sav = Feedback;
+	known_feedbacks = num_feedbacks;
 
 	f = sav;
-	for (i = 0; i < *num_feedbacks; i++) {
+	for (i = 0; i < num_feedbacks; i++) {
+	    Bool skip = False;
 	    switch (f->class) {
 	    case KbdFeedbackClass:
 	    {
@@ -257,14 +266,18 @@ XGetFeedbackControl(
 		break;
 	    }
 	    default:
+		--known_feedbacks;
+		skip = True;
 		break;
 	    }
 	    f = (xFeedbackState *) ((char *)f + f->length);
-	    Feedback = (XFeedbackState *) ((char *)Feedback + Feedback->length);
+	    if (!skip)
+		Feedback = (XFeedbackState *) ((char *)Feedback + Feedback->length);
 	}
     }
 out:
     XFree((char *)sav);
+    *num_feedbacks_out = known_feedbacks;
 
     UnlockDisplay(dpy);
     SyncHandle();
